#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: device_replacement_info
short_description: Information module for Device Replacement Info
description:
- This module represents an alias of the module device_replacement_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  faultyDeviceName:
    description:
    - FaultyDeviceName query parameter. Faulty Device Name.
    type: str
  faultyDevicePlatform:
    description:
    - FaultyDevicePlatform query parameter. Faulty Device Platform.
    type: str
  replacementDevicePlatform:
    description:
    - ReplacementDevicePlatform query parameter. Replacement Device Platform.
    type: str
  faultyDeviceSerialNumber:
    description:
    - FaultyDeviceSerialNumber query parameter. Faulty Device Serial Number.
    type: str
  replacementDeviceSerialNumber:
    description:
    - ReplacementDeviceSerialNumber query parameter. Replacement Device Serial Number.
    type: str
  replacementStatus:
    description:
    - >
      ReplacementStatus query parameter. Device Replacement status READY-FOR-REPLACEMENT, REPLACEMENT-IN-PROGRESS,
      REPLACEMENT-SCHEDULED, REPLACED, ERROR, NETWORK_READINESS_REQUESTED, NETWORK_READINESS_FAILED.
    elements: str
    type: list
  family:
    description:
    - Family query parameter. List of familiesRouters, Switches and Hubs, AP.
    elements: str
    type: list
  sortBy:
    description:
    - SortBy query parameter. SortBy this field. SortBy is mandatory when order is used.
    type: str
  sortOrder:
    description:
    - SortOrder query parameter. Order on displayNameASC,DESC.
    type: str
  offset:
    description:
    - Offset query parameter.
    type: int
  limit:
    description:
    - Limit query parameter.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Replacement ReturnListOfReplacementDevicesWithReplacementDetailsV1
  description: Complete reference of the ReturnListOfReplacementDevicesWithReplacementDetailsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!return-list-of-replacement-devices-with-replacement-details
notes:
  - SDK Method used are
    device_replacement.DeviceReplacement.return_replacement_devices_with_details,

  - Paths used are
    get /dna/intent/api/v1/device-replacement,
  - It should be noted that this module is an alias of device_replacement_v1_info

"""

EXAMPLES = r"""
- name: Get all Device Replacement Info
  cisco.dnac.device_replacement_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    faultyDeviceName: string
    faultyDevicePlatform: string
    replacementDevicePlatform: string
    faultyDeviceSerialNumber: string
    replacementDeviceSerialNumber: string
    replacementStatus: []
    family: []
    sortBy: string
    sortOrder: string
    offset: 0
    limit: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "creationTime": 0,
          "family": "string",
          "faultyDeviceId": "string",
          "faultyDeviceName": "string",
          "faultyDevicePlatform": "string",
          "faultyDeviceSerialNumber": "string",
          "id": "string",
          "neighbourDeviceId": "string",
          "networkReadinessTaskId": "string",
          "replacementDevicePlatform": "string",
          "replacementDeviceSerialNumber": "string",
          "replacementStatus": "string",
          "replacementTime": 0,
          "workflowId": "string",
          "workflowFailedStep": "string",
          "readinesscheckTaskId": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_profiles_v1
short_description: Resource module for Wireless Profiles V1
description:
- Manage operations create, update and delete of the resource Wireless Profiles V1.
- This API allows the user to create a Wireless Network Profile.
- This API allows the user to delete Wireless Network Profile by ID.
- This API allows the user to update a Wireless Network Profile by ID.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Wireless Profile Id.
    type: str
  ssidDetails:
    description: Wireless Profiles's ssidDetails.
    elements: dict
    suboptions:
      dot11beProfileId:
        description: 802.11be Profile Id. Applicable to IOS controllers with version
          17.15 and higher. 802.11be Profiles if passed, should be same across all SSIDs
          in network profile being configured.
        type: str
      enableFabric:
        description: True if fabric is enabled, else False. Flex and fabric cannot be
          enabled simultaneously and a profile can only contain either flex SSIDs or
          fabric SSIDs and not both at the same time.
        type: bool
      flexConnect:
        description: Wireless Profiles's flexConnect.
        suboptions:
          enableFlexConnect:
            description: True if flex connect is enabled, else False. Flex and fabric
              cannot be enabled simultaneously and a profile can only contain either
              flex SSIDs or fabric SSIDs and not both at the same time.
            type: bool
          localToVlan:
            description: Local to VLAN ID.
            type: int
        type: dict
      interfaceName:
        description: Interface Name. Default Value management.
        type: str
      ssidName:
        description: SSID Name.
        type: str
      wlanProfileName:
        description: WLAN Profile Name.
        type: str
    type: list
  wirelessProfileName:
    description: Wireless Network Profile Name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless CreateWirelessProfileConnectivityV1
  description: Complete reference of the CreateWirelessProfileConnectivityV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-wireless-profile-connectivity
- name: Cisco DNA Center documentation for Wireless DeleteWirelessProfileConnectivityV1
  description: Complete reference of the DeleteWirelessProfileConnectivityV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-wireless-profile-connectivity
- name: Cisco DNA Center documentation for Wireless UpdateWirelessProfileConnectivityV1
  description: Complete reference of the UpdateWirelessProfileConnectivityV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-wireless-profile-connectivity
notes:
  - SDK Method used are
    wireless.Wireless.create_wireless_profile_connectivity_v1,
    wireless.Wireless.delete_wireless_profile_connectivity_v1,
    wireless.Wireless.update_wireless_profile_connectivity_v1,

  - Paths used are
    post /dna/intent/api/v1/wirelessProfiles,
    delete /dna/intent/api/v1/wirelessProfiles/{id},
    put /dna/intent/api/v1/wirelessProfiles/{id},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_profiles_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    ssidDetails:
    - dot11beProfileId: string
      enableFabric: true
      flexConnect:
        enableFlexConnect: true
        localToVlan: 0
      interfaceName: string
      ssidName: string
      wlanProfileName: string
    wirelessProfileName: string

- name: Update by id
  cisco.dnac.wireless_profiles_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    id: string
    ssidDetails:
    - dot11beProfileId: string
      enableFabric: true
      flexConnect:
        enableFlexConnect: true
        localToVlan: 0
      interfaceName: string
      ssidName: string
      wlanProfileName: string
    wirelessProfileName: string

- name: Delete by id
  cisco.dnac.wireless_profiles_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_port_assignment_for_user_device_v1
short_description: Resource module for Sda Port Assignment For User Device V1
description:
- Manage operations create and delete of the resource Sda Port Assignment For User Device V1.
- Add Port assignment for user device in SDA Fabric.
- Delete Port assignment for user device in SDA Fabric.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  authenticateTemplateName:
    description: Authenticate TemplateName associated with siteNameHierarchy.
    type: str
    version_added: 4.0.0
  dataIpAddressPoolName:
    description: Ip Pool Name, that is assigned to virtual network with traffic type
      as DATA(can't be empty if voiceIpAddressPoolName is empty).
    type: str
    version_added: 4.0.0
  deviceManagementIpAddress:
    description: DeviceManagementIpAddress query parameter.
    type: str
  interfaceDescription:
    description: User defined text message for port assignment.
    type: str
    version_added: 4.0.0
  interfaceName:
    description: InterfaceName query parameter.
    type: str
  interfaceNames:
    description: List of Interface Names on the Edge Node Device. E.g."GigabitEthernet1/0/3","GigabitEthernet1/0/4".
    elements: str
    type: list
  scalableGroupName:
    description: Scalable Group name associated with VN.
    type: str
    version_added: 4.0.0
  siteNameHierarchy:
    description: Complete Path of SD-Access Fabric Site.
    type: str
    version_added: 4.0.0
  voiceIpAddressPoolName:
    description: Ip Pool Name, that is assigned to virtual network with traffic type
      as VOICE(can't be empty if dataIpAddressPoolName is empty).
    type: str
    version_added: 4.0.0
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddPortAssignmentForUserDeviceInSDAFabricV1
  description: Complete reference of the AddPortAssignmentForUserDeviceInSDAFabricV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-port-assignment-for-user-device-in-sda-fabric
- name: Cisco DNA Center documentation for SDA DeletePortAssignmentForUserDeviceInSDAFabricV1
  description: Complete reference of the DeletePortAssignmentForUserDeviceInSDAFabricV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-port-assignment-for-user-device-in-sda-fabric
notes:
  - SDK Method used are
    sda.Sda.add_port_assignment_for_user_device,
    sda.Sda.delete_port_assignment_for_user_device,

  - Paths used are
    post /dna/intent/api/v1/business/sda/hostonboarding/user-device,
    delete /dna/intent/api/v1/business/sda/hostonboarding/user-device,

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.sda_port_assignment_for_user_device_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    deviceManagementIpAddress: string
    interfaceName: string

- name: Create
  cisco.dnac.sda_port_assignment_for_user_device_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    authenticateTemplateName: string
    dataIpAddressPoolName: string
    deviceManagementIpAddress: string
    interfaceDescription: string
    interfaceName: string
    interfaceNames:
    - string
    scalableGroupName: string
    siteNameHierarchy: string
    voiceIpAddressPoolName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "taskId": "string",
      "taskStatusUrl": "string",
      "executionStatusUrl": "string",
      "executionId": "string"
    }
"""



"""
json parser

This is the json parser for use with the cli_parse module and action plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    author: Bradley Thornton (@cidrblock)
    name: json
    short_description: Define configurable options for B(json) sub-plugin of M(ansible.utils.cli_parse) module
    description:
    - This plugin documentation provides the configurable options that can be passed
      to the M(ansible.utils.cli_parse) plugins when I(ansible.utils.json) is used as a value for
      I(name) option.
    version_added: 1.0.0
"""

EXAMPLES = r"""
- name: "Run command and parse with json"
  ansible.utils.cli_parse:
    command: "show version | json"
    parser:
      name: ansible.utils.json
  register: nxos_json_command

- name: "Load text and parse with json"
  ansible.utils.cli_parse:
    text: "{{ lookup('ansible.builtin.file', './nxos_show_interface_json_text.txt') }}"
    parser:
      name: ansible.utils.json
  register: nxos_json_text
"""

import json

from ansible.module_utils._text import to_native
from ansible.module_utils.six import string_types

from ansible_collections.ansible.utils.plugins.plugin_utils.base.cli_parser import CliParserBase


class CliParser(CliParserBase):
    """The json parser class
    Convert a string containing valid json into an object
    """

    DEFAULT_TEMPLATE_EXTENSION = None
    PROVIDE_TEMPLATE_CONTENTS = False

    def parse(self, *_args, **_kwargs):
        """Std entry point for a cli_parse parse execution

        :return: Errors or parsed text as structured data
        :rtype: dict

        :example:

        The parse function of a parser should return a dict:
        {"errors": [a list of errors]}
        or
        {"parsed": obj}
        """
        text = self._task_args.get("text")
        try:
            if not isinstance(text, string_types):
                text = json.dumps(text)
            parsed = json.loads(text)
        except Exception as exc:
            return {"errors": [to_native(exc)]}

        return {"parsed": parsed}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: app_policy_intent_create_v1
short_description: Resource module for App Policy Intent Create V1
description:
- Manage operation create of the resource App Policy Intent Create V1.
- Create/Update/Delete application policy.
version_added: '4.0.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  createList:
    description: App Policy Intent Create's createList.
    elements: dict
    suboptions:
      advancedPolicyScope:
        description: App Policy Intent Create's advancedPolicyScope.
        suboptions:
          advancedPolicyScopeElement:
            description: App Policy Intent Create's advancedPolicyScopeElement.
            elements: dict
            suboptions:
              groupId:
                description: The site(s) ID where the Application QoS Policy will be
                  deployed.
                elements: str
                type: list
              ssid:
                description: Ssid.
                elements: str
                type: list
            type: list
          name:
            description: Policy name.
            type: str
        type: dict
      consumer:
        description: App Policy Intent Create's consumer.
        suboptions:
          scalableGroup:
            description: App Policy Intent Create's scalableGroup.
            elements: dict
            suboptions:
              idRef:
                description: Id ref to application Scalable group.
                type: str
            type: list
        type: dict
      contract:
        description: App Policy Intent Create's contract.
        suboptions:
          idRef:
            description: Id ref to Queueing profile.
            type: str
        type: dict
      deletePolicyStatus:
        description: NONE deployed policy to devices, DELETED delete policy from devices,
          RESTORED restored to original configuration.
        type: str
      exclusiveContract:
        description: App Policy Intent Create's exclusiveContract.
        suboptions:
          clause:
            description: App Policy Intent Create's clause.
            elements: dict
            suboptions:
              deviceRemovalBehavior:
                description: Device eemoval behavior.
                type: str
              hostTrackingEnabled:
                description: Is host tracking enabled.
                type: bool
              relevanceLevel:
                description: Relevance level.
                type: str
              type:
                description: Type.
                type: str
            type: list
        type: dict
      name:
        description: Concatination of <polcy name>_<application-set-name> or <polcy
          name>_global_policy_configuration or <polcy name>_queuing_customization.
        type: str
      policyScope:
        description: Policy name.
        type: str
      priority:
        description: Set to 4095 while producer refer to application Scalable group
          otherwise 100.
        type: str
      producer:
        description: App Policy Intent Create's producer.
        suboptions:
          scalableGroup:
            description: App Policy Intent Create's scalableGroup.
            elements: dict
            suboptions:
              idRef:
                description: Id ref to application-set or application Scalable group.
                type: str
            type: list
        type: dict
    type: list
  deleteList:
    description: Delete list of Group Based Policy ids.
    elements: str
    type: list
  updateList:
    description: App Policy Intent Create's updateList.
    elements: dict
    suboptions:
      advancedPolicyScope:
        description: App Policy Intent Create's advancedPolicyScope.
        suboptions:
          advancedPolicyScopeElement:
            description: App Policy Intent Create's advancedPolicyScopeElement.
            elements: dict
            suboptions:
              groupId:
                description: The site(s) ID where the Application QoS Policy will be
                  deployed.
                elements: str
                type: list
              id:
                description: Id of Advance policy scope element.
                type: str
              ssid:
                description: Ssid.
                elements: str
                type: list
            type: list
          id:
            description: Id of Advance policy scope.
            type: str
          name:
            description: Policy name.
            type: str
        type: dict
      consumer:
        description: App Policy Intent Create's consumer.
        suboptions:
          id:
            description: Id of Consumer.
            type: str
          scalableGroup:
            description: App Policy Intent Create's scalableGroup.
            elements: dict
            suboptions:
              idRef:
                description: Id ref to application Scalable group.
                type: str
            type: list
        type: dict
      contract:
        description: App Policy Intent Create's contract.
        suboptions:
          idRef:
            description: Id ref to Queueing profile.
            type: str
        type: dict
      deletePolicyStatus:
        description: NONE deployed policy to devices, DELETED delete policy from devices,
          RESTORED restored to original configuration.
        type: str
      exclusiveContract:
        description: App Policy Intent Create's exclusiveContract.
        suboptions:
          clause:
            description: App Policy Intent Create's clause.
            elements: dict
            suboptions:
              deviceRemovalBehavior:
                description: Device removal behavior.
                type: str
              hostTrackingEnabled:
                description: Host tracking enabled.
                type: bool
              id:
                description: Id of Business relevance or Application policy knobs clause.
                type: str
              relevanceLevel:
                description: Relevance level.
                type: str
              type:
                description: Type.
                type: str
            type: list
          id:
            description: Id of Exclusive contract.
            type: str
        type: dict
      id:
        description: Id of Group based policy.
        type: str
      name:
        description: Concatination of <polcy name>_<application-set-name> or <polcy
          name>_global_policy_configuration or <polcy name>_queuing_customization.
        type: str
      policyScope:
        description: Policy name.
        type: str
      priority:
        description: Set to 4095 while producer refer to application Scalable group
          otherwise 100.
        type: str
      producer:
        description: App Policy Intent Create's producer.
        suboptions:
          id:
            description: Id of Producer.
            type: str
          scalableGroup:
            description: App Policy Intent Create's scalableGroup.
            elements: dict
            suboptions:
              idRef:
                description: Id ref to application-set or application Scalable group.
                type: str
            type: list
        type: dict
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Application Policy ApplicationPolicyIntentV1
  description: Complete reference of the ApplicationPolicyIntentV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!application-policy-intent
notes:
  - SDK Method used are
    application_policy.ApplicationPolicy.application_policy_intent_v1,

  - Paths used are
    post /dna/intent/api/v1/app-policy-intent,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.app_policy_intent_create_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    createList:
    - advancedPolicyScope:
        advancedPolicyScopeElement:
        - groupId:
          - string
          ssid:
          - string
        name: string
      consumer:
        scalableGroup:
        - idRef: string
      contract:
        idRef: string
      deletePolicyStatus: string
      exclusiveContract:
        clause:
        - deviceRemovalBehavior: string
          hostTrackingEnabled: true
          relevanceLevel: string
          type: string
      name: string
      policyScope: string
      priority: string
      producer:
        scalableGroup:
        - idRef: string
    deleteList:
    - string
    updateList:
    - advancedPolicyScope:
        advancedPolicyScopeElement:
        - groupId:
          - string
          id: string
          ssid:
          - string
        id: string
        name: string
      consumer:
        id: string
        scalableGroup:
        - idRef: string
      contract:
        idRef: string
      deletePolicyStatus: string
      exclusiveContract:
        clause:
        - deviceRemovalBehavior: string
          hostTrackingEnabled: true
          id: string
          relevanceLevel: string
          type: string
        id: string
      id: string
      name: string
      policyScope: string
      priority: string
      producer:
        id: string
        scalableGroup:
        - idRef: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: lan_automation_update_device
short_description: Resource module for Lan Automation Update Device
description:
- This module represents an alias of the module lan_automation_update_device_v1
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  feature:
    description: Feature query parameter. Feature ID for the update. Supported feature
      IDs include LOOPBACK0_IPADDRESS_UPDATE, HOSTNAME_UPDATE, LINK_ADD, and LINK_DELETE.
    type: str
  hostnameUpdateDevices:
    description: Lan Automation Update Device's hostnameUpdateDevices.
    elements: dict
    suboptions:
      deviceManagementIPAddress:
        description: Device Management IP Address.
        type: str
      newHostName:
        description: New hostname for the device.
        type: str
    type: list
  linkUpdate:
    description: Lan Automation Update Device's linkUpdate.
    suboptions:
      destinationDeviceInterfaceName:
        description: Destination Device Interface Name.
        type: str
      destinationDeviceManagementIPAddress:
        description: Destination Device Management IP Address.
        type: str
      ipPoolName:
        description: Name of the IP LAN Pool, required for Link Add should be from discovery
          site of source and destination device.
        type: str
      sourceDeviceInterfaceName:
        description: Source Device Interface Name.
        type: str
      sourceDeviceManagementIPAddress:
        description: Source Device Management IP Address.
        type: str
    type: dict
  loopbackUpdateDeviceList:
    description: Lan Automation Update Device's loopbackUpdateDeviceList.
    elements: dict
    suboptions:
      deviceManagementIPAddress:
        description: Device Management IP Address.
        type: str
      newLoopback0IPAddress:
        description: New Loopback0 IP Address from LAN Pool of Device Discovery Site(Shared
          pool should not be used).
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for LAN Automation LANAutomationDeviceUpdateV1
  description: Complete reference of the LANAutomationDeviceUpdateV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!l-an-automation-device-update
notes:
  - SDK Method used are
    lan_automation.LanAutomation.lan_automation_device_update_v1,

  - Paths used are
    put /dna/intent/api/v1/lan-automation/updateDevice,
  - It should be noted that this module is an alias of lan_automation_update_device_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.lan_automation_update_device:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    feature: string
    hostnameUpdateDevices:
    - deviceManagementIPAddress: string
      newHostName: string
    linkUpdate:
      destinationDeviceInterfaceName: string
      destinationDeviceManagementIPAddress: string
      ipPoolName: string
      sourceDeviceInterfaceName: string
      sourceDeviceManagementIPAddress: string
    loopbackUpdateDeviceList:
    - deviceManagementIPAddress: string
      newLoopback0IPAddress: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_vpn_info
version_added: 1.0.0
version_added_collection: community.aws
short_description: Gather information about EC2 VPN Connections in AWS
description:
  - Gather information about EC2 VPN Connections in AWS.
author:
  - Madhura Naniwadekar (@Madhura-CSI)
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeVpnConnections.html) for possible filters.
    required: false
    type: dict
    default: {}
  vpn_connection_ids:
    description:
      - Get details of specific EC2 VPN Connection(s) using vpn connection ID/IDs. This value should be provided as a list.
    required: false
    type: list
    elements: str
    default: []
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# # Note: These examples do not set authentication details, see the AWS Guide for details.
- name: Gather information about all  EC2 VPN Connections
  amazon.aws.ec2_vpc_vpn_info:

- name: Gather information about a filtered list of  EC2 VPN Connections, based on tags
  amazon.aws.ec2_vpc_vpn_info:
    filters:
      "tag:Name": "test-connection"
  register: vpn_conn_info

- name: Gather information about EC2 VPN Connections by specifying connection IDs
  amazon.aws.ec2_vpc_vpn_info:
    filters:
      "vpn-gateway-id": "vgw-cbe66beb"
  register: vpn_conn_info
"""

RETURN = r"""
vpn_connections:
    description: List of one or more EC2 VPN Connections.
    type: list
    elements: dict
    returned: always
    contains:
      category:
        description: The category of the VPN connection.
        returned: always
        type: str
        sample: "VPN"
      customer_gatway_configuration:
        description: The configuration information for the VPN connection's customer gateway (in the native XML format).
        returned: always
        type: str
      customer_gateway_id:
        description: The ID of the customer gateway at your end of the VPN connection.
        returned: always
        type: str
        sample: "cgw-17a53c37"
      gateway_association_state:
        description: The current state of the gateway association.
        type: str
        sample: "associated"
      options:
        description: The VPN connection options.
        type: list
        elements: dict
        contains:
          static_routes_only:
            description: If the VPN connection only allows static routes.
            type: bool
            sample: true
          enable_acceleration:
            description: Indicates whether acceleration is enabled for the VPN connection.
            type: bool
            sample: false
          local_ipv4_network_cidr:
            description: The IPv4 CIDR on the customer gateway (on-premises) side of the VPN connection.
            type: str
            sample: "0.0.0.0/0"
          outside_ip_address_type:
            description: The external IP address of the VPN tunnel.
            type: str
            sample: "PublicIpv4"
          remote_ipv4_network_cidr:
            description: The IPv4 CIDR on the Amazon Web Services side of the VPN connection.
            type: str
            sample: "0.0.0.0/0"
          tunnel_inside_ip_version:
            description: Indicates whether the VPN tunnels process IPv4 or IPv6 traffic.
            type: str
            sample: "ipv4"
          tunnel_options:
            description: Indicates the VPN tunnel options.
            type: list
            elements: dict
            sample: [
                  {
                      "log_options": {
                          "cloud_watch_log_options": {
                              "log_enabled": false
                          }
                      },
                      "outside_ip_address": "34.225.101.10",
                      "pre_shared_key": "8n7hnjNE8zhIt4VpMOIfcrw6XnUTHLW9",
                      "tunnel_inside_cidr": "169.254.31.8/30"
                  },
              ]
            contains:
              log_options:
                description: Options for logging VPN tunnel activity.
                type: dict
                contains:
                  cloud_watch_log_options:
                    description: Options for sending VPN tunnel logs to CloudWatch.
                    type: dict
              outside_ip_address:
                description: The external IP address of the VPN tunnel.
                type: str
              pre_shared_key:
                description:
                  - The pre-shared key (PSK) to establish initial authentication between the
                    virtual private gateway and the customer gateway.
                type: str
              tunnel_inside_cidr:
                description: The range of inside IPv4 addresses for the tunnel.
                type: str
      routes:
        description: List of static routes associated with the VPN connection.
        returned: always
        type: list
        elements: dict
        contains:
          destination_cidr_block:
            description:
              - The CIDR block associated with the local subnet of the customer data center.
            type: str
          source:
            description: Indicates how the routes were provided.
            type: str
          state:
            description: The current state of the static route.
            type: str
      state:
        description: The current state of the VPN connection.
        returned: always
        type: str
        sample: "available"
      tags:
        description: Any tags assigned to the VPN connection.
        returned: always
        type: dict
        sample: {
                  "Name": "test-conn"
                }
      type:
        description: The type of VPN connection.
        returned: always
        type: str
        sample: "ipsec.1"
      vgw_telemetry:
         description: Information about the VPN tunnel.
         returned: always
         type: dict
         contains:
           accepted_route_count:
             description: The number of accepted routes.
             returned: always
             type: int
             sample: 0
           last_status_change:
               description: The date and time of the last change in status.
               returned: always
               type: str
               sample: "2018-02-09T14:35:27+00:00"
           outside_ip_address:
               description: The Internet-routable IP address of the virtual private gateway's outside interface.
               returned: always
               type: str
               sample: "13.127.79.191"
           status:
               description: The status of the VPN tunnel.
               returned: always
               type: str
               sample: "DOWN"
           status_message:
               description: If an error occurs, a description of the error.
               returned: always
               type: str
               sample: "IPSEC IS DOWN"
           certificate_arn:
               description: The Amazon Resource Name of the virtual private gateway tunnel endpoint certificate.
               returned: when a private certificate is used for authentication
               type: str
               sample: "arn:aws:acm:us-east-1:123456789012:certificate/c544d8ce-20b8-4fff-98b0-example"
      vpn_connection_id:
        description: The ID of the VPN connection.
        returned: always
        type: str
        sample: "vpn-f700d5c0"
      vpn_gateway_id:
        description: The ID of the virtual private gateway at the AWS side of the VPN connection.
        returned: always
        type: str
        sample: "vgw-cbe56bfb"
"""

import json
from typing import Any
from typing import Dict
from typing import NoReturn

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_vpn_connections
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def date_handler(obj: Dict[str, Any]) -> Dict[str, Any]:
    return obj.isoformat() if hasattr(obj, "isoformat") else obj


def list_vpn_connections(client, module: AnsibleAWSModule) -> NoReturn:
    params: Dict[str, Any] = {}

    params["Filters"] = ansible_dict_to_boto3_filter_list(module.params.get("filters"))
    params["VpnConnectionIds"] = module.params.get("vpn_connection_ids")

    try:
        result = json.loads(json.dumps(describe_vpn_connections(client, **params), default=date_handler))
    except ValueError as e:
        module.fail_json(e, msg="Cannot validate JSON data")
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg="Could not describe customer gateways")

    snaked_vpn_connections = [camel_dict_to_snake_dict(vpn_connection) for vpn_connection in result]
    if snaked_vpn_connections:
        for vpn_connection in snaked_vpn_connections:
            vpn_connection["tags"] = boto3_tag_list_to_ansible_dict(vpn_connection.get("tags", []))

    module.exit_json(changed=False, vpn_connections=snaked_vpn_connections)


def main():
    argument_spec = dict(
        vpn_connection_ids=dict(default=[], type="list", elements="str"),
        filters=dict(default={}, type="dict"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        mutually_exclusive=[["vpn_connection_ids", "filters"]],
        supports_check_mode=True,
    )

    connection = module.client("ec2")

    list_vpn_connections(connection, module)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The iosxr legacy fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""


from __future__ import absolute_import, division, print_function


__metaclass__ = type


import platform
import re

from ansible.module_utils.six import iteritems
from ansible.module_utils.six.moves import zip

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.iosxr import (
    get_capabilities,
    run_commands,
)


class FactsBase(object):
    COMMANDS = frozenset()

    def __init__(self, module):
        self.module = module
        self.facts = dict()
        self.warnings = list()
        self.responses = None

    def populate(self):
        self.responses = run_commands(
            self.module,
            list(self.COMMANDS),
            check_rc=False,
        )


class Default(FactsBase):
    def populate(self):
        self.facts.update(self.platform_facts())

    def platform_facts(self):
        platform_facts = {}

        resp = get_capabilities(self.module)
        device_info = resp["device_info"]

        platform_facts["system"] = device_info["network_os"]
        for item in ("serialnum", "model", "image", "version", "platform", "hostname"):
            val = device_info.get("network_os_%s" % item)
            if val:
                platform_facts[item] = val

        platform_facts["api"] = resp["network_api"]
        platform_facts["python_version"] = platform.python_version()

        return platform_facts


class Hardware(FactsBase):
    COMMANDS = ["dir /all", "show memory summary", "show processes cpu | include CPU utilization"]

    def populate(self):
        super(Hardware, self).populate()

        data = self.responses[0]
        self.facts["filesystems"] = self.parse_filesystems(data)
        self.facts["cpu_utilization"] = self.parse_cpu_utilization(self.responses[2])

        data = self.responses[1]
        match = re.search(r"Physical Memory: (\d+)M total \((\d+)", data)
        if match:
            self.facts["memtotal_mb"] = match.group(1)
            self.facts["memfree_mb"] = match.group(2)

    def parse_filesystems(self, data):
        return re.findall(r"^Directory of (\S+)", data, re.M)

    def parse_cpu_utilization(self, data):
        facts = {}
        cpu_utilization_regex = re.compile(
            r"""
            ^CPU\sutilization\sfor\sone\sminute:(\s(?P<one_min>[0-9]+)?%)?;
            \sfive\sminutes:\s(?P<five_mins>[0-9]+)?%;
            \sfifteen\sminutes:\s(?P<fifteen_mins>[0-9]+)?%
            """,
            re.VERBOSE,
        )

        for line in data.split("\n"):
            cpu_utilization_match = cpu_utilization_regex.match(line)

            if cpu_utilization_match:
                facts["one_minute"] = int(cpu_utilization_match.group("one_min"))
                facts["five_minutes"] = int(cpu_utilization_match.group("five_mins"))
                facts["fifteen_minutes"] = int(cpu_utilization_match.group("fifteen_mins"))

        return facts


class Config(FactsBase):
    COMMANDS = ["show running-config"]

    def populate(self):
        super(Config, self).populate()
        self.facts["config"] = self.responses[0]


class Interfaces(FactsBase):
    COMMANDS = [
        "show interfaces",
        "show ipv6 interface",
        "show lldp",
        "show lldp neighbors detail",
        "show cdp",
        "show cdp neighbors detail",
    ]

    def populate(self):
        super(Interfaces, self).populate()
        self.facts["all_ipv4_addresses"] = list()
        self.facts["all_ipv6_addresses"] = list()

        interfaces = self.parse_interfaces(self.responses[0])
        self.facts["interfaces"] = self.populate_interfaces(interfaces)

        data = self.responses[1]
        if len(data) > 0:
            data = self.parse_interfaces(data)
            self.populate_ipv6_interfaces(data)

        if "LLDP is not enabled" not in self.responses[2]:
            neighbors = self.responses[3]
            self.facts["neighbors"] = self.parse_neighbors(neighbors)

        if "CDP is not enabled" not in self.responses[4]:
            neighbors = self.responses[5]
            self.facts["neighbors"] = self.parse_cdp_neighbors(neighbors)

    def populate_interfaces(self, interfaces):
        facts = dict()
        for key, value in iteritems(interfaces):
            intf = dict()
            intf["description"] = self.parse_description(value)
            intf["macaddress"] = self.parse_macaddress(value)

            ipv4 = self.parse_ipv4(value)
            intf["ipv4"] = self.parse_ipv4(value)
            if ipv4:
                self.add_ip_address(ipv4["address"], "ipv4")

            intf["mtu"] = self.parse_mtu(value)
            intf["bandwidth"] = self.parse_bandwidth(value)
            intf["duplex"] = self.parse_duplex(value)
            intf["lineprotocol"] = self.parse_lineprotocol(value)
            intf["operstatus"] = self.parse_operstatus(value)
            intf["type"] = self.parse_type(value)

            facts[key] = intf
        return facts

    def populate_ipv6_interfaces(self, data):
        for key, value in iteritems(data):
            if key in ["No", "RPF"] or key.startswith("IP"):
                continue
            self.facts["interfaces"][key]["ipv6"] = list()
            addresses = re.findall(r"\s+(.+), subnet", value, re.M)
            subnets = re.findall(r", subnet is (.+)$", value, re.M)
            for addr, subnet in zip(addresses, subnets):
                ipv6 = dict(address=addr.strip(), subnet=subnet.strip())
                self.add_ip_address(addr.strip(), "ipv6")
                self.facts["interfaces"][key]["ipv6"].append(ipv6)

    def add_ip_address(self, address, family):
        if family == "ipv4":
            self.facts["all_ipv4_addresses"].append(address)
        else:
            self.facts["all_ipv6_addresses"].append(address)

    def parse_neighbors(self, neighbors):
        facts = dict()
        nbors = neighbors.split(
            "------------------------------------------------",
        )
        for entry in nbors[1:]:
            if entry == "":
                continue
            intf = self.parse_lldp_intf(entry)
            if intf not in facts:
                facts[intf] = list()
            fact = dict()
            fact["host"] = self.parse_lldp_host(entry)
            fact["remote_description"] = self.parse_lldp_remote_desc(entry)
            fact["port"] = self.parse_lldp_port(entry)
            facts[intf].append(fact)
        return facts

    def parse_cdp_neighbors(self, neighbors):
        facts = dict()
        for entry in neighbors.split("-------------------------"):
            if entry == "":
                continue
            intf_port = self.parse_cdp_intf_port(entry)
            if intf_port is None:
                return facts
            intf, port = intf_port
            if intf not in facts:
                facts[intf] = list()
            fact = dict()
            fact["host"] = self.parse_cdp_host(entry)
            fact["platform"] = self.parse_cdp_platform(entry)
            fact["port"] = port
            fact["ip"] = self.parse_cdp_ip(entry)
            facts[intf].append(fact)
        return facts

    def parse_interfaces(self, data):
        parsed = dict()
        key = ""
        for line in data.split("\n"):
            if len(line) == 0:
                continue
            if line[0] == " ":
                parsed[key] += "\n%s" % line
            else:
                match = re.match(r"^(\S+)", line)
                if match:
                    key = match.group(1)
                    parsed[key] = line
        return parsed

    def parse_description(self, data):
        match = re.search(r"Description: (.+)$", data, re.M)
        if match:
            return match.group(1)

    def parse_macaddress(self, data):
        match = re.search(r"address is (\S+)", data)
        if match:
            return match.group(1)

    def parse_ipv4(self, data):
        match = re.search(r"Internet address is (\S+)/(\d+)", data)
        if match:
            addr = match.group(1)
            masklen = int(match.group(2))
            return dict(address=addr, masklen=masklen)

    def parse_mtu(self, data):
        match = re.search(r"MTU (\d+)", data)
        if match:
            return int(match.group(1))

    def parse_bandwidth(self, data):
        match = re.search(r"BW (\d+)", data)
        if match:
            return int(match.group(1))

    def parse_duplex(self, data):
        match = re.search(r"(\w+)(?: D|-d)uplex", data, re.M)
        if match:
            return match.group(1)

    def parse_type(self, data):
        match = re.search(r"Hardware is (.+),", data, re.M)
        if match:
            return match.group(1)

    def parse_lineprotocol(self, data):
        match = re.search(r"line protocol is (.+)\s+?$", data, re.M)
        if match:
            return match.group(1)

    def parse_operstatus(self, data):
        match = re.search(r"^(?:.+) is (.+),", data, re.M)
        if match:
            return match.group(1)

    def parse_lldp_intf(self, data):
        match = re.search(r"^Local Interface: (.+)$", data, re.M)
        if match:
            return match.group(1)

    def parse_lldp_remote_desc(self, data):
        match = re.search(r"Port Description: (.+)$", data, re.M)
        if match:
            return match.group(1)

    def parse_lldp_host(self, data):
        match = re.search(r"System Name: (.+)$", data, re.M)
        if match:
            return match.group(1)

    def parse_lldp_port(self, data):
        match = re.search(r"Port id: (.+)$", data, re.M)
        if match:
            return match.group(1)

    def parse_cdp_intf_port(self, data):
        match = re.search(r"^Interface: (.+),  Port ID \(outgoing port\): (.+)$", data, re.M)
        if match:
            return match.group(1), match.group(2)

    def parse_cdp_host(self, data):
        match = re.search(r"^Device ID: (.+)$", data, re.M)
        if match:
            return match.group(1)

    def parse_cdp_platform(self, data):
        match = re.search(r"^Platform: (.+),", data, re.M)
        if match:
            return match.group(1)

    def parse_cdp_ip(self, data):
        match = re.search(r"^  IP address: (.+)$", data, re.M)
        if match:
            return match.group(1)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: file_namespace_files_v1_info
short_description: Information module for File Namespace Files V1
description:
- Get File Namespace Files V1 by name.
- Returns list of files under a specific namespace.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  nameSpace:
    description:
    - NameSpace path parameter. A listing of fileId's.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for File GetListOfFilesV1
  description: Complete reference of the GetListOfFilesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-list-of-files
notes:
  - SDK Method used are
    file.File.get_list_of_files_v1,

  - Paths used are
    get /dna/intent/api/v1/file/namespace/{nameSpace},

"""

EXAMPLES = r"""
- name: Get File Namespace Files V1 by name
  cisco.dnac.file_namespace_files_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    nameSpace: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "attributeInfo": {},
          "downloadPath": "string",
          "encrypted": true,
          "fileFormat": "string",
          "fileSize": "string",
          "id": "string",
          "md5Checksum": "string",
          "name": "string",
          "nameSpace": "string",
          "sftpServerList": [
            {}
          ],
          "sha1Checksum": "string",
          "taskId": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2016 Matt Davis, <mdavis@ansible.com>
#                    Chris Houseknecht, <house@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_storageblob
short_description: Manage blob containers and blob objects
version_added: "0.0.1"
description:
    - Create, update and delete blob containers and blob objects.
    - Use to upload a file and store it as a blob object, or download a blob object to a file(upload and download mode)
    - Use to upload a batch of files under a given directory(batch upload mode)
    - In the batch upload mode, the existing blob object will be overwritten if a blob object with the same name is to be created.
    - the module can work exclusively in three modes, when C(batch_upload_src) is set, it is working in batch upload mode;
      when C(src) is set, it is working in upload mode and when C(dst) is set, it is working in dowload mode.
options:
    auth_mode:
        description:
            - The mode in which to run the command. C(login) mode will directly use your login credentials for the authentication.
            - The legacy C(key) mode will attempt to query for an account key if no authentication parameters for the account are provided.
            - Can also be set via the environment variable C(AZURE_STORAGE_AUTH_MODE).
        default: key
        type: str
        choices:
            - key
            - login
        version_added: "1.19.0"
    storage_account_name:
        description:
            - Name of the storage account to use.
        required: true
        type: str
        aliases:
            - account_name
            - storage_account
    blob:
        description:
            - Name of a blob object within the container.
        aliases:
            - blob_name
        type: str
    blob_type:
        description:
            - Type of blob object.
        default: block
        type: str
        choices:
            - block
            - page
        version_added: "0.0.1"
    container:
        description:
            - Name of a blob container within the storage account.
        required: true
        type: str
        aliases:
            - container_name
    content_type:
        description:
            - Set the blob content-type header. For example C(image/png).
        type: str
    cache_control:
        description:
            - Set the blob cache-control header.
        type: str
    content_disposition:
        description:
            - Set the blob content-disposition header.
        type: str
    content_encoding:
        description:
            - Set the blob encoding header.
        type: str
    content_language:
        description:
            - Set the blob content-language header.
        type: str
    content_md5:
        description:
            - Set the blob md5 hash value.
        type: str
    dest:
        description:
            - Destination file path. Use with state C(present) to download a blob.
        aliases:
            - destination
        type: path
    force:
        description:
            - Overwrite existing blob or file when uploading or downloading. Force deletion of a container that contains blobs.
        type: bool
        default: no
    resource_group:
        description:
            - Name of the resource group to use.
        required: true
        type: str
        aliases:
            - resource_group_name
    src:
        description:
            - Source file path. Use with state C(present) to upload a blob.
        type: str
        aliases:
            - source
    batch_upload_src:
        description:
            - Batch upload source directory. Use with state C(present) to upload batch of files under the directory.
        type: path
    batch_upload_dst:
        description:
            - Base directory in container when upload batch of files.
        type: path
    standard_blob_tier:
        description:
            - Specifies the blob tier to set the blob to.
            - This is only applicable for block blobs on standard storage accounts.
        type: str
        choices:
            - Archive
            - Cool
            - Cold
            - Hot
    state:
        description:
            - State of a container or blob.
            - Use state C(absent) with a container value only to delete a container. Include a blob value to remove
              a specific blob. A container will not be deleted, if it contains blobs. Use the I(force) option to override,
              deleting the container and all associated blobs.
            - Use state C(present) to create or update a container and upload or download a blob. If the container
              does not exist, it will be created. If it exists, it will be updated with configuration options. Provide
              a blob name and either src or dest to upload or download. Provide a src path to upload and a dest path
              to download. If a blob (uploading) or a file (downloading) already exists, it will not be overwritten
              unless I(force=true).
        default: present
        type: str
        choices:
            - absent
            - present
    public_access:
        description:
            - A container's level of public access. By default containers are private.
            - Can only be set at time of container creation.
            - The I(public_access) parameter should not be set for private access.
            - If I(public_access) is not configured, access level is by default private. It can not be set explicitely to private.
        type: str
        choices:
            - container
            - blob

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Chris Houseknecht (@chouseknecht)
    - Matt Davis (@nitzmahone)

'''

EXAMPLES = '''
- name: Remove container foo
  azure_rm_storageblob:
    resource_group: myResourceGroup
    storage_account_name: clh0002
    container: foo
    state: absent

- name: Create container foo and upload a file
  azure_rm_storageblob:
    resource_group: myResourceGroup
    storage_account_name: clh0002
    container: foo
    blob: graylog.png
    src: ./files/graylog.png
    public_access: container
    content_type: 'application/image'

- name: Download the file
  azure_rm_storageblob:
    resource_group: myResourceGroup
    storage_account_name: clh0002
    container: foo
    blob: graylog.png
    dest: ~/tmp/images/graylog.png
'''

RETURN = '''
blob:
    description:
        - Facts about the current state of the blob.
    returned: when a blob is operated on
    type: dict
    sample: {
        "content_length": 136532,
        "content_settings": {
            "cache_control": null,
            "content_disposition": null,
            "content_encoding": null,
            "content_language": null,
            "content_md5": null,
            "content_type": "application/image"
        },
        "last_modified": "09-Mar-2016 22:08:25 +0000",
        "name": "graylog.png",
        "tags": {},
        "type": "BlockBlob",
        'standard_blob_tier': 'Hot'
    }
container:
    description:
        - Facts about the current state of the selected container.
    returned: always
    type: dict
    sample: {
        "last_modified": "09-Mar-2016 19:28:26 +0000",
        "name": "foo",
        "tags": {}
    }
'''

import os
import mimetypes

try:
    from azure.storage.blob._models import BlobType, ContentSettings, StandardBlobTier
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible.module_utils.basic import env_fallback


class AzureRMStorageBlob(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            auth_mode=dict(
                type='str',
                choices=['key', 'login'],
                fallback=(env_fallback, ['AZURE_STORAGE_AUTH_MODE']),
                default="key"
            ),
            storage_account_name=dict(required=True, type='str', aliases=['account_name', 'storage_account']),
            blob=dict(type='str', aliases=['blob_name']),
            blob_type=dict(type='str', default='block', choices=['block', 'page']),
            standard_blob_tier=dict(type='str', choices=['Archive', 'Cool', 'Cold', 'Hot']),
            container=dict(required=True, type='str', aliases=['container_name']),
            dest=dict(type='path', aliases=['destination']),
            force=dict(type='bool', default=False),
            resource_group=dict(required=True, type='str', aliases=['resource_group_name']),
            src=dict(type='str', aliases=['source']),
            batch_upload_src=dict(type='path'),
            batch_upload_dst=dict(type='path'),
            state=dict(type='str', default='present', choices=['absent', 'present']),
            public_access=dict(type='str', choices=['container', 'blob']),
            content_type=dict(type='str'),
            content_encoding=dict(type='str'),
            content_language=dict(type='str'),
            content_disposition=dict(type='str'),
            cache_control=dict(type='str'),
            content_md5=dict(type='str'),
        )

        mutually_exclusive = [('src', 'dest'), ('src', 'batch_upload_src'), ('dest', 'batch_upload_src')]

        self.blob_service_client = None
        self.blob_details = None
        self.storage_account_name = None
        self.blob = None
        self.blob_obj = None
        self.blob_type = None
        self.standard_blob_tier = None
        self.container = None
        self.container_obj = None
        self.dest = None
        self.force = None
        self.resource_group = None
        self.src = None
        self.batch_upload_src = None
        self.batch_upload_dst = None
        self.state = None
        self.tags = None
        self.public_access = None
        self.results = dict(
            changed=False,
            actions=[],
            container=dict(),
            blob=dict()
        )

        super(AzureRMStorageBlob, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 mutually_exclusive=mutually_exclusive,
                                                 supports_tags=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        self.results['check_mode'] = self.check_mode

        # add file path validation

        self.blob_service_client = self.get_blob_service_client(self.resource_group, self.storage_account_name, self.auth_mode)
        self.container_obj = self.get_container()
        if self.blob:
            self.blob_obj = self.get_blob()

        if self.state == 'present':
            if not self.container_obj:
                # create the container
                self.create_container()
            elif self.container_obj and not self.blob:
                # update container attributes
                update_tags, self.container_obj['tags'] = self.update_tags(self.container_obj.get('tags'))
                if update_tags:
                    self.update_container_tags(self.container_obj['tags'])

            if self.batch_upload_src:
                self.batch_upload()
                return self.results

            if self.blob:
                # create, update or download blob
                if self.src and self.src_is_valid():
                    if self.blob_obj and not self.force:
                        self.log("Cannot upload to {0}. Blob with that name already exists. Use the force option".format(self.blob))
                    else:
                        self.upload_blob()
                elif self.dest and self.dest_is_valid():
                    self.download_blob()

                update_tags, self.blob_obj['tags'] = self.update_tags(self.blob_obj.get('tags'))
                if update_tags:
                    self.update_blob_tags(self.blob_obj['tags'])

                if self.blob_content_settings_differ():
                    self.update_blob_content_settings()

                if self.standard_blob_tier is not None and self.blob_obj.get('standard_blob_tier') is not None and \
                   self.blob_obj['standard_blob_tier'] != self.standard_blob_tier:
                    self.update_blob_tier()

        elif self.state == 'absent':
            if self.container_obj and not self.blob:
                # Delete container
                if self.container_has_blobs():
                    if self.force:
                        self.delete_container()
                    else:
                        self.log("Cannot delete container {0}. It contains blobs. Use the force option.".format(
                            self.container))
                else:
                    self.delete_container()
            elif self.container_obj and self.blob_obj:
                # Delete blob
                self.delete_blob()

        # until we sort out how we want to do this globally
        del self.results['actions']
        return self.results

    def batch_upload(self):

        def _glob_files_locally(folder_path):

            len_folder_path = len(folder_path) + 1

            for root, v, files in os.walk(folder_path):
                for f in files:
                    full_path = os.path.join(root, f)
                    yield full_path, full_path[len_folder_path:]

        def _normalize_blob_file_path(path, name):
            path_sep = '/'
            if path:
                name = path_sep.join((path, name))

            return path_sep.join(os.path.normpath(name).split(os.path.sep)).strip(path_sep)

        def _guess_content_type(file_path, original):
            if original.content_encoding or original.content_type:
                return original

            mimetypes.add_type('application/json', '.json')
            mimetypes.add_type('application/javascript', '.js')
            mimetypes.add_type('application/wasm', '.wasm')

            content_type, v = mimetypes.guess_type(file_path)
            return ContentSettings(content_type=content_type,
                                   content_disposition=original.content_disposition,
                                   content_language=original.content_language,
                                   content_md5=original.content_md5,
                                   cache_control=original.cache_control)

        if not os.path.exists(self.batch_upload_src):
            self.fail("batch upload source source directory {0} does not exist".format(self.batch_upload_src))

        if not os.path.isdir(self.batch_upload_src):
            self.fail("incorrect usage: {0} is not a directory".format(self.batch_upload_src))

        source_dir = os.path.realpath(self.batch_upload_src)
        source_files = list(_glob_files_locally(source_dir))

        content_settings = ContentSettings(content_type=self.content_type,
                                           content_encoding=self.content_encoding,
                                           content_language=self.content_language,
                                           content_disposition=self.content_disposition,
                                           cache_control=self.cache_control,
                                           content_md5=None)

        for src, blob_path in source_files:
            if self.batch_upload_dst:
                blob_path = _normalize_blob_file_path(self.batch_upload_dst, blob_path)
            if not self.check_mode:
                try:
                    client = self.blob_service_client.get_blob_client(container=self.container, blob=blob_path)
                    with open(src, "rb") as data:
                        client.upload_blob(data=data,
                                           blob_type=self.get_blob_type(self.blob_type),
                                           metadata=self.tags,
                                           content_settings=_guess_content_type(src, content_settings),
                                           standard_blob_tier=self.get_blob_tier(self.standard_blob_tier),
                                           overwrite=self.force)
                except Exception as exc:
                    self.fail("Error creating blob {0} - {1}".format(src, str(exc)))
            self.results['actions'].append('created blob from {0}'.format(src))

        self.results['changed'] = True
        self.results['container'] = self.container_obj

    def get_blob_tier(self, blob_tier):
        if blob_tier == "archive":
            return StandardBlobTier.Archive
        elif blob_tier == "cool":
            return StandardBlobTier.Cool
        elif blob_tier == "hot":
            return StandardBlobTier.Hot
        elif blob_tier == "cold":
            return StandardBlobTier.Cold
        else:
            return None

    def get_blob_type(self, blob_type):
        if blob_type == "block":
            return BlobType.BlockBlob
        elif blob_type == "page":
            return BlobType.PageBlob
        else:
            return BlobType.AppendBlob

    def get_container(self):
        result = {}
        container = None
        if self.container:
            try:
                container = self.blob_service_client.get_container_client(container=self.container).get_container_properties()
            except ResourceNotFoundError:
                pass
        if container:
            result = dict(
                name=container["name"],
                tags=container["metadata"],
                last_modified=container["last_modified"].strftime('%d-%b-%Y %H:%M:%S %z'),
            )
        return result

    def get_blob(self):
        result = dict()
        blob = None
        if self.blob:
            try:
                blob = self.blob_service_client.get_blob_client(container=self.container, blob=self.blob).get_blob_properties()
            except ResourceNotFoundError:
                pass
        if blob:
            result = dict(
                name=blob["name"],
                tags=blob["metadata"],
                last_modified=blob["last_modified"].strftime('%d-%b-%Y %H:%M:%S %z'),
                type=blob["blob_type"],
                standard_blob_tier=blob.get('blob_tier'),
                content_length=blob["size"],
                content_settings=dict(
                    content_type=blob["content_settings"]["content_type"],
                    content_encoding=blob["content_settings"]["content_encoding"],
                    content_language=blob["content_settings"]["content_language"],
                    content_disposition=blob["content_settings"]["content_disposition"],
                    cache_control=blob["content_settings"]["cache_control"],
                    content_md5=blob["content_settings"]["content_md5"].hex() if blob["content_settings"]["content_md5"] else None,
                )
            )
        return result

    def create_container(self):
        self.log('Create container %s' % self.container)

        tags = None
        if not self.blob and self.tags:
            # when a blob is present, then tags are assigned at the blob level
            tags = self.tags

        if not self.check_mode:
            try:
                client = self.blob_service_client.get_container_client(container=self.container)
                client.create_container(metadata=tags, public_access=self.public_access)
            except Exception as exc:
                self.fail("Error creating container {0} - {1}".format(self.container, str(exc)))
        self.container_obj = self.get_container()
        self.results['changed'] = True
        self.results['actions'].append('created container {0}'.format(self.container))
        self.results['container'] = self.container_obj

    def upload_blob(self):
        content_settings = None
        if self.content_type or self.content_encoding or self.content_language or self.content_disposition or \
                self.cache_control or self.content_md5:
            content_settings = ContentSettings(
                content_type=self.content_type,
                content_encoding=self.content_encoding,
                content_language=self.content_language,
                content_disposition=self.content_disposition,
                cache_control=self.cache_control,
                content_md5=self.content_md5
            )
        if not self.check_mode:
            try:
                client = self.blob_service_client.get_blob_client(container=self.container, blob=self.blob)
                with open(self.src, "rb") as data:
                    client.upload_blob(data=data,
                                       blob_type=self.get_blob_type(self.blob_type),
                                       metadata=self.tags,
                                       content_settings=content_settings,
                                       standard_blob_tier=self.get_blob_tier(self.standard_blob_tier),
                                       overwrite=self.force)
            except Exception as exc:
                self.fail("Error creating blob {0} - {1}".format(self.blob, str(exc)))

        self.blob_obj = self.get_blob()
        self.results['changed'] = True
        self.results['actions'].append('created blob {0} from {1}'.format(self.blob, self.src))
        self.results['container'] = self.container_obj
        self.results['blob'] = self.blob_obj

    def download_blob(self):
        if not self.check_mode:
            try:
                client = self.blob_service_client.get_blob_client(container=self.container, blob=self.blob)
                with open(self.dest, "wb") as blob_stream:
                    blob_data = client.download_blob()
                    blob_data.readinto(blob_stream)
            except Exception as exc:
                self.fail("Failed to download blob {0}:{1} to {2} - {3}".format(self.container,
                                                                                self.blob,
                                                                                self.dest,
                                                                                exc))
        self.results['changed'] = True
        self.results['actions'].append('downloaded blob {0}:{1} to {2}'.format(self.container,
                                                                               self.blob,
                                                                               self.dest))

        self.results['container'] = self.container_obj
        self.results['blob'] = self.blob_obj

    def src_is_valid(self):
        if not os.path.isfile(self.src):
            self.fail("The source path must be a file.")
        if os.access(self.src, os.R_OK):
            return True
        self.fail("Failed to access {0}. Make sure the file exists and that you have "
                  "read access.".format(self.src))

    def dest_is_valid(self):
        if not self.check_mode:
            if not os.path.basename(self.dest):
                # dest is a directory
                if os.path.isdir(self.dest):
                    self.log("Path is dir. Appending blob name.")
                    self.dest += self.blob
                else:
                    try:
                        self.log('Attempting to makedirs {0}'.format(self.dest))
                        os.makedirs(self.dest)
                    except IOError as exc:
                        self.fail("Failed to create directory {0} - {1}".format(self.dest, str(exc)))
                    self.dest += self.blob
            else:
                # does path exist without basename
                file_name = os.path.basename(self.dest)
                path = self.dest.replace(file_name, '')
                self.log('Checking path {0}'.format(path))
                if not os.path.isdir(path):
                    try:
                        self.log('Attempting to makedirs {0}'.format(path))
                        os.makedirs(path)
                    except IOError as exc:
                        self.fail("Failed to create directory {0} - {1}".format(path, str(exc)))
            self.log('Checking final path {0}'.format(self.dest))
            if os.path.isfile(self.dest) and not self.force:
                # dest already exists and we're not forcing
                self.log("Dest {0} already exists. Cannot download. Use the force option.".format(self.dest))
                return False
        return True

    def delete_container(self):
        if not self.check_mode:
            try:
                self.blob_service_client.get_container_client(container=self.container).delete_container()
            except Exception as exc:
                self.fail("Error deleting container {0} - {1}".format(self.container, str(exc)))

        self.results['changed'] = True
        self.results['actions'].append('deleted container {0}'.format(self.container))

    def container_has_blobs(self):
        try:
            blobs = self.blob_service_client.get_container_client(container=self.container).list_blobs()
        except Exception as exc:
            self.fail("Error list blobs in {0} - {1}".format(self.container, str(exc)))
        if len(list(blobs)) > 0:
            return True
        return False

    def delete_blob(self):
        if not self.check_mode:
            try:
                self.blob_service_client.get_container_client(container=self.container).delete_blob(blob=self.blob)
            except Exception as exc:
                self.fail("Error deleting blob {0}:{1} - {2}".format(self.container, self.blob, str(exc)))

        self.results['changed'] = True
        self.results['actions'].append('deleted blob {0}:{1}'.format(self.container, self.blob))
        self.results['container'] = self.container_obj

    def update_container_tags(self, tags):
        if not self.check_mode:
            try:
                self.blob_service_client.get_container_client(container=self.container).set_container_metadata(metadata=tags)
            except Exception as exc:
                self.fail("Error updating container tags {0} - {1}".format(self.container, str(exc)))
        self.container_obj = self.get_container()
        self.results['changed'] = True
        self.results['actions'].append("updated container {0} tags.".format(self.container))
        self.results['container'] = self.container_obj

    def update_blob_tags(self, tags):
        if not self.check_mode:
            try:
                self.blob_service_client.get_blob_client(container=self.container, blob=self.blob).set_blob_metadata(metadata=tags)
            except Exception as exc:
                self.fail("Update blob tags {0}:{1} - {2}".format(self.container, self.blob, str(exc)))
        self.blob_obj = self.get_blob()
        self.results['changed'] = True
        self.results['actions'].append("updated blob {0}:{1} tags.".format(self.container, self.blob))
        self.results['container'] = self.container_obj
        self.results['blob'] = self.blob_obj

    def update_blob_tier(self):
        client = self.blob_service_client.get_blob_client(container=self.container, blob=self.blob)
        client.set_standard_blob_tier(standard_blob_tier=self.standard_blob_tier)
        self.results['changed'] = True

    def blob_content_settings_differ(self):
        if self.content_type or self.content_encoding or self.content_language or self.content_disposition or \
                self.cache_control or self.content_md5:
            settings = dict(
                content_type=self.content_type,
                content_encoding=self.content_encoding,
                content_language=self.content_language,
                content_disposition=self.content_disposition,
                cache_control=self.cache_control,
                content_md5=self.content_md5
            )
            if self.blob_obj['content_settings'] != settings:
                return True

        return False

    def update_blob_content_settings(self):
        content_settings = ContentSettings(
            content_type=self.content_type,
            content_encoding=self.content_encoding,
            content_language=self.content_language,
            content_disposition=self.content_disposition,
            cache_control=self.cache_control,
            content_md5=self.content_md5
        )
        if not self.check_mode:
            try:
                self.blob_service_client.get_blob_client(container=self.container, blob=self.blob).set_http_headers(content_settings=content_settings)
            except Exception as exc:
                self.fail("Update blob content settings {0}:{1} - {2}".format(self.container, self.blob, str(exc)))

        self.blob_obj = self.get_blob()
        self.results['changed'] = True
        self.results['actions'].append("updated blob {0}:{1} content settings.".format(self.container, self.blob))
        self.results['container'] = self.container_obj
        self.results['blob'] = self.blob_obj


def main():
    AzureRMStorageBlob()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_snapshot
version_added: "0.1.2"
short_description: Manage Azure Snapshot instance
description:
    - Create, update and delete instance of Azure Snapshot.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    name:
        description:
            - Resource name.
        required: true
        type: str
    location:
        description:
            - Resource location.
        type: str
    incremental:
        description:
            - Whether a snapshot is incremental.
            - Incremental snapshots on the same disk occupy less space than full snapshots and can be diffed.
        type: bool
        default: False
    sku:
        description:
            - The snapshots SKU.
        type: dict
        suboptions:
            name:
                description:
                    - The sku name.
                type: str
                choices:
                    - Standard_LRS
                    - Premium_LRS
                    - Standard_ZRS
            tier:
                description:
                    - The sku tier.
                type: str
    os_type:
        description:
            - The Operating System type.
        type: str
        choices:
            - Linux
            - Windows
    creation_data:
        description:
            - Disk source information.
            - CreationData information cannot be changed after the disk has been created.
        type: dict
        suboptions:
            create_option:
                description:
                    - This enumerates the possible sources of a disk's creation.
                type: str
                choices:
                    - Import
                    - Copy
            source_uri:
                description:
                    - If I(create_option=Import), this is the URI of a blob to be imported into a managed disk.
                type: str
            source_id:
                description:
                    - If I(create_option=Copy), this is the resource ID of a managed disk to be copied from.
                type: str
    state:
        description:
            - Assert the state of the Snapshot.
            - Use C(present) to create or update an Snapshot and C(absent) to delete it.
        default: present
        type: str
        choices:
          - absent
          - present
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Create a snapshot by importing an unmanaged blob from the same subscription.
  azure_rm_snapshot:
    resource_group: myResourceGroup
    name: mySnapshot
    location: eastus
    creation_data:
      create_option: Import
      source_uri: 'https://mystorageaccount.blob.core.windows.net/osimages/osimage.vhd'

- name: Create a snapshot by copying an existing managed disk.
  azure_rm_snapshot:
    resource_group: myResourceGroup
    name: mySnapshot
    location: eastus
    creation_data:
      create_option: Copy
      source_id: '/subscriptions/sub123/resourceGroups/group123/providers/Microsoft.Compute/disks/disk123'
'''

RETURN = '''
id:
    description:
        - Resource ID.
    returned: always
    type: str
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Compute/snapshots/mySnapshot
'''

import time
import json
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMSnapshots(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            sku=dict(
                type='dict',
                options=dict(
                    name=dict(
                        type='str',
                        choices=['Standard_LRS',
                                 'Premium_LRS',
                                 'Standard_ZRS']
                    ),
                    tier=dict(
                        type='str'
                    )
                )
            ),
            os_type=dict(
                type='str',
                choices=['Windows',
                         'Linux']
            ),
            incremental=dict(type='bool', default=False),
            creation_data=dict(
                type='dict',
                options=dict(
                    create_option=dict(
                        type='str',
                        choices=['Import', 'Copy'],
                    ),
                    source_uri=dict(
                        type='str'
                    ),
                    source_id=dict(
                        type='str'
                    )
                )
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.id = None
        self.name = None
        self.type = None
        self.managed_by = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200, 201, 202]
        self.to_do = Actions.NoAction

        self.body = {}
        self.body['properties'] = dict()
        self.query_parameters = {}
        self.query_parameters['api-version'] = '2022-03-02'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(AzureRMSnapshots, self).__init__(derived_arg_spec=self.module_arg_spec,
                                               supports_check_mode=True,
                                               supports_tags=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                if key == 'incremental':
                    self.body['properties']['incremental'] = kwargs[key]
                elif key == 'os_type':
                    self.body['properties']['osType'] = kwargs[key]
                elif key == 'creation_data':
                    self.body['properties']['creationData'] = dict()
                    if kwargs[key].get('create_option') is not None:
                        self.body['properties']['creationData']['createOption'] = kwargs[key].get('create_option')
                        self.body['properties']['creationData']['sourceUri'] = kwargs[key].get('source_uri')
                        self.body['properties']['creationData']['sourceResourceId'] = kwargs[key].get('source_id')
                        if kwargs[key].get('source_uri') is not None:
                            self.query_parameters['api-version'] = '2019-03-01'
                else:
                    self.body[key] = kwargs[key]

        old_response = None
        response = None

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        resource_group = self.get_resource_group(self.resource_group)

        if 'location' not in self.body:
            self.body['location'] = resource_group.location

        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.Compute' +
                    '/snapshots' +
                    '/{{ snapshot_name }}')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)
        self.url = self.url.replace('{{ snapshot_name }}', self.name)

        old_response = self.get_resource()

        if not old_response:
            self.log("Snapshot instance doesn't exist")

            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log('Snapshot instance already exists')

            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                if self.body.get('sku') is not None and \
                   not all(self.body['sku'][item] == old_response['sku'].get(item) for item in self.body['sku'].keys()):
                    self.to_do = Actions.Update
                if self.body['properties'].get('incremental') is not None and \
                   self.body['properties']['incremental'] != old_response['properties']['incremental']:
                    self.to_do = Actions.Update
                if self.body['properties'].get('osType') is not None and \
                   self.body['properties']['osType'] != old_response['properties'].get('osType'):
                    self.to_do = Actions.Update
                if self.body['properties'].get('creationData') is not None and \
                   not all(self.body['properties']['creationData'][item] == old_response['properties']['creationData'].get(item)
                   for item in self.body['properties']['creationData'].keys()):
                    self.to_do = Actions.Update

                update_tags, self.body['tags'] = self.update_tags(old_response.get('tags'))
                if update_tags:
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log('Need to Create / Update the Snapshot instance')

            if self.check_mode:
                self.results['changed'] = True
                return self.results
            response = self.create_update_resource()
            self.results['changed'] = True
            self.log('Creation / Update done')
        elif self.to_do == Actions.Delete:
            self.log('Snapshot instance deleted')
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_resource()

            # make sure instance is actually deleted, for some Azure resources, instance is hanging around
            # for some time after deletion -- this should be really fixed in Azure
            while self.get_resource():
                time.sleep(20)
        else:
            self.log('Snapshot instance unchanged')
            self.results['changed'] = False
            response = old_response

        if response:
            self.results["id"] = response["id"]

        return self.results

    def create_update_resource(self):
        # self.log('Creating / Updating the Snapshot instance {0}'.format(self.))
        response = None
        try:
            response = self.mgmt_client.query(url=self.url,
                                              method='PUT',
                                              query_parameters=self.query_parameters,
                                              header_parameters=self.header_parameters,
                                              body=self.body,
                                              expected_status_codes=self.status_code,
                                              polling_timeout=600,
                                              polling_interval=30)
        except Exception as exc:
            self.log('Error attempting to create the Snapshot instance.')
            self.fail('Error creating the Snapshot instance: {0}'.format(str(exc)))

        if hasattr(response, 'body'):
            response = json.loads(response.body())
        elif hasattr(response, 'context'):
            response = response.context['deserialized_data']
        else:
            self.fail("Create or Updating fail, no match message return, return info as {0}".format(response))

        return response

    def delete_resource(self):
        # self.log('Deleting the Snapshot instance {0}'.format(self.))
        try:
            response = self.mgmt_client.query(url=self.url,
                                              method='DELETE',
                                              query_parameters=self.query_parameters,
                                              header_parameters=self.header_parameters,
                                              body=None,
                                              expected_status_codes=self.status_code,
                                              polling_timeout=600,
                                              polling_interval=30)
        except Exception as e:
            self.log('Error attempting to delete the Snapshot instance.')
            self.fail('Error deleting the Snapshot instance: {0}'.format(str(e)))

        return True

    def get_resource(self):
        # self.log('Checking if the Snapshot instance {0} is present'.format(self.))
        found = False
        try:
            response = self.mgmt_client.query(url=self.url,
                                              method='GET',
                                              query_parameters=self.query_parameters,
                                              header_parameters=self.header_parameters,
                                              body=None,
                                              expected_status_codes=self.status_code,
                                              polling_timeout=600,
                                              polling_interval=30)
            response = json.loads(response.body())
            found = True
            self.log("Response : {0}".format(response))
            # self.log("Snapshot instance : {0} found".format(response.name))
        except Exception as e:
            self.log('Did not find the Snapshot instance.')
        if found is True:
            return response

        return False


def main():
    AzureRMSnapshots()


if __name__ == '__main__':
    main()
